@@ -15,7 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -426,8 +427,7 @@ dsa_sign( int algo, MPI *resarr, MPI data, MPI *skey )
 }
 
 int
-dsa_verify( int algo, MPI hash, MPI *data, MPI *pkey,
-		    int (*cmp)(void *, MPI), void *opaquev )
+dsa_verify( int algo, MPI hash, MPI *data, MPI *pkey )
 {
     DSA_public_key pk;
 
@@ -479,5 +479,3 @@ dsa_get_info( int algo, int *npkey, int *nskey, int *nenc, int *nsig,
       default: *use = 0; return NULL;
     }
 }
-
-
