@@ -15,7 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  *
  * ***********************************************************************
  * The algorithm used to generate practically save primes is due to
@@ -462,15 +463,16 @@ is_prime( MPI n, int steps, int *count )
 	    mpi_set_ui( x, 2 );
 	}
 	else {
-	    /*mpi_set_bytes( x, nbits-1, get_random_byte, 0 );*/
-	    {	char *p = get_random_bits( nbits, 0, 0 );
-		mpi_set_buffer( x, p, (nbits+7)/8, 0 );
-		m_free(p);
-	    }
-	    /* make sure that the number is smaller than the prime
-	     * and keep the randomness of the high bit */
+            char *p;
+            
+            p = get_random_bits( nbits, 0, 0 );
+            mpi_set_buffer( x, p, (nbits+7)/8, 0 );
+            m_free(p);
+
+	    /* Make sure that the number is smaller than the prime
+	     * and keep the randomness of the high bit.  */
 	    if( mpi_test_bit( x, nbits-2 ) ) {
-		mpi_set_highbit( x, nbits-2 ); /* clear all higher bits */
+		mpi_set_highbit( x, nbits-2 ); /* Clear all higher bits */
 	    }
 	    else {
 		mpi_set_highbit( x, nbits-2 );
@@ -498,6 +500,7 @@ is_prime( MPI n, int steps, int *count )
     mpi_free( z );
     mpi_free( nminus1 );
     mpi_free( q );
+    mpi_free (a2);
 
     return rc;
 }
