@@ -16,7 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  *
  *************************************************************************
  * The code here is based on code from Cryptlib 3.0 beta by Peter Gutmann.
@@ -654,13 +655,14 @@ rndw32_gather_random_fast( void (*add)(const void*, size_t, int), int requester
 	(*add) ( &kernelTime, sizeof (kernelTime), requester );
 	(*add) ( &userTime, sizeof (userTime), requester );
 
-	/* Get the minimum and maximum working set size for the current process */
+	/* Get the minimum and maximum working set size for the
+           current process */
 	GetProcessWorkingSetSize (handle, &minimumWorkingSetSize,
 					  &maximumWorkingSetSize);
 	(*add) ( &minimumWorkingSetSize,
-				   sizeof (&minimumWorkingSetSize), requester );
+				   sizeof (minimumWorkingSetSize), requester );
 	(*add) ( &maximumWorkingSetSize,
-				   sizeof (&maximumWorkingSetSize), requester );
+				   sizeof (maximumWorkingSetSize), requester );
     }
 
 
@@ -688,7 +690,7 @@ rndw32_gather_random_fast( void (*add)(const void*, size_t, int), int requester
 	if (QueryPerformanceCounter (&performanceCount)) {
 	    if ( debug_me )
 		log_debug ("rndw32#gather_random_fast: perf data\n");
-	    (*add) (&performanceCount, sizeof (&performanceCount), requester);
+	    (*add) (&performanceCount, sizeof (performanceCount), requester);
 	}
 	else { /* Millisecond accuracy at best... */
 	    DWORD aword = GetTickCount ();
