@@ -15,9 +15,10 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  *
- * $Id: apdu.c,v 1.18 2004/12/15 05:16:53 dshaw Exp $
+ * $Id: apdu.c,v 1.21 2005/05/31 08:39:17 wk Exp $
  */
 
 #include <config.h>
@@ -65,11 +66,16 @@
 #include "dynload.h"
 #include "ccid-driver.h"
 
+
+/* To to conflicting use of threading libraries we usually can't link
+   against libpcsclite.   Instead we use a wrapper program.  */
 #ifdef USE_GNU_PTH
+#ifndef HAVE_W32_SYSTEM
 #define NEED_PCSC_WRAPPER 1
 #endif
+#endif
 
-
+ 
 #define MAX_READER 4 /* Number of readers we support concurrently. */
 
 
@@ -145,12 +151,12 @@ static struct reader_table_s reader_table[MAX_READER];
 
 
 /* ct API function pointer. */
-static char (* DLSTDCALL CT_init) (unsigned short ctn, unsigned short Pn);
-static char (* DLSTDCALL CT_data) (unsigned short ctn, unsigned char *dad,
+static char (DLSTDCALL * CT_init) (unsigned short ctn, unsigned short Pn);
+static char (DLSTDCALL * CT_data) (unsigned short ctn, unsigned char *dad,
                                    unsigned char *sad, unsigned short lc,
                                    unsigned char *cmd, unsigned short *lr,
                                    unsigned char *rsp);
-static char (* DLSTDCALL CT_close) (unsigned short ctn);
+static char (DLSTDCALL * CT_close) (unsigned short ctn);
 
 /* PC/SC constants and function pointer. */
 #define PCSC_SCOPE_USER      0 
@@ -212,46 +218,46 @@ struct pcsc_readerstate_s
 
 typedef struct pcsc_readerstate_s *pcsc_readerstate_t;
 
-long (* DLSTDCALL pcsc_establish_context) (unsigned long scope,
+long (DLSTDCALL * pcsc_establish_context) (unsigned long scope,
                                            const void *reserved1,
                                            const void *reserved2,
                                            unsigned long *r_context);
-long (* DLSTDCALL pcsc_release_context) (unsigned long context);
-long (* DLSTDCALL pcsc_list_readers) (unsigned long context,
+long (DLSTDCALL * pcsc_release_context) (unsigned long context);
+long (DLSTDCALL * pcsc_list_readers) (unsigned long context,
                                       const char *groups,
                                       char *readers, unsigned long*readerslen);
-long (* DLSTDCALL pcsc_get_status_change) (unsigned long context,
+long (DLSTDCALL * pcsc_get_status_change) (unsigned long context,
                                            unsigned long timeout,
                                            pcsc_readerstate_t readerstates,
                                            unsigned long nreaderstates);
-long (* DLSTDCALL pcsc_connect) (unsigned long context,
+long (DLSTDCALL * pcsc_connect) (unsigned long context,
                                  const char *reader,
                                  unsigned long share_mode,
                                  unsigned long preferred_protocols,
                                  unsigned long *r_card,
                                  unsigned long *r_active_protocol);
-long (* DLSTDCALL pcsc_reconnect) (unsigned long card,
+long (DLSTDCALL * pcsc_reconnect) (unsigned long card,
                                    unsigned long share_mode,
                                    unsigned long preferred_protocols,
                                    unsigned long initialization,
                                    unsigned long *r_active_protocol);
-long (* DLSTDCALL pcsc_disconnect) (unsigned long card,
+long (DLSTDCALL * pcsc_disconnect) (unsigned long card,
                                     unsigned long disposition);
-long (* DLSTDCALL pcsc_status) (unsigned long card,
+long (DLSTDCALL * pcsc_status) (unsigned long card,
                                 char *reader, unsigned long *readerlen,
                                 unsigned long *r_state,
                                 unsigned long *r_protocol,
                                 unsigned char *atr, unsigned long *atrlen);
-long (* DLSTDCALL pcsc_begin_transaction) (unsigned long card);
-long (* DLSTDCALL pcsc_end_transaction) (unsigned long card);
-long (* DLSTDCALL pcsc_transmit) (unsigned long card,
+long (DLSTDCALL * pcsc_begin_transaction) (unsigned long card);
+long (DLSTDCALL * pcsc_end_transaction) (unsigned long card);
+long (DLSTDCALL * pcsc_transmit) (unsigned long card,
                                   const pcsc_io_request_t send_pci,
                                   const unsigned char *send_buffer,
                                   unsigned long send_len,
                                   pcsc_io_request_t recv_pci,
                                   unsigned char *recv_buffer,
                                   unsigned long *recv_len);
-long (* DLSTDCALL pcsc_set_timeout) (unsigned long context,
+long (DLSTDCALL * pcsc_set_timeout) (unsigned long context,
                                      unsigned long timeout);
 
 
@@ -1482,7 +1488,6 @@ open_pcsc_reader (const char *portstr)
     }
   strcpy (reader_table[slot].rdrname, portstr? portstr : list);
   xfree (list);
-  list = NULL;
 
   err = pcsc_connect (reader_table[slot].pcsc.context,
                       reader_table[slot].rdrname,
@@ -1496,11 +1501,11 @@ open_pcsc_reader (const char *portstr)
     {
       log_error ("pcsc_connect failed: %s (0x%lx)\n",
                   pcsc_error_string (err), err);
-  
-    pcsc_release_context (reader_table[slot].pcsc.context);
+      pcsc_release_context (reader_table[slot].pcsc.context);
       xfree (reader_table[slot].rdrname);
       reader_table[slot].rdrname = NULL;
       reader_table[slot].used = 0;
+      xfree (list);
       return -1;
     }      
 
@@ -2717,8 +2722,8 @@ apdu_send_le(int slot, int class, int ins, int p0, int p1,
   resultlen -= 2;
   if (DBG_CARD_IO)
     {
-      log_debug (" response: sw=%04X  datalen=%u\n",
-		 sw, (unsigned int)resultlen);
+      log_debug (" response: sw=%04X  datalen=%d\n",
+                 sw, (unsigned int)resultlen);
       if ( !retbuf && (sw == SW_SUCCESS || (sw & 0xff00) == SW_MORE_DATA))
         log_printhex ("     dump: ", result, resultlen);
     }
@@ -2784,8 +2789,8 @@ apdu_send_le(int slot, int class, int ins, int p0, int p1,
           resultlen -= 2;
           if (DBG_CARD_IO)
             {
-              log_debug ("     more: sw=%04X  datalen=%u\n",
-			 sw, (unsigned int)resultlen);
+              log_debug ("     more: sw=%04X  datalen=%d\n",
+                         sw, (unsigned int)resultlen);
               if (!retbuf && (sw==SW_SUCCESS || (sw&0xff00)==SW_MORE_DATA))
                 log_printhex ("     dump: ", result, resultlen);
             }
@@ -2918,8 +2923,8 @@ apdu_send_direct (int slot, const unsigned char *apdudata, size_t apdudatalen,
   resultlen -= 2;
   if (DBG_CARD_IO)
     {
-      log_debug (" response: sw=%04X  datalen=%u\n",
-		 sw, (unsigned int)resultlen);
+      log_debug (" response: sw=%04X  datalen=%d\n",
+                 sw, (unsigned int)resultlen);
       if ( !retbuf && (sw == SW_SUCCESS || (sw & 0xff00) == SW_MORE_DATA))
         log_printhex ("     dump: ", result, resultlen);
     }
@@ -2971,8 +2976,8 @@ apdu_send_direct (int slot, const unsigned char *apdudata, size_t apdudatalen,
           resultlen -= 2;
           if (DBG_CARD_IO)
             {
-              log_debug ("     more: sw=%04X  datalen=%u\n",
-			 sw, (unsigned int)resultlen);
+              log_debug ("     more: sw=%04X  datalen=%d\n",
+                         sw, (unsigned int)resultlen);
               if (!retbuf && (sw==SW_SUCCESS || (sw&0xff00)==SW_MORE_DATA))
                 log_printhex ("     dump: ", result, resultlen);
             }
