@@ -16,7 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -789,7 +790,7 @@ list_keyblock_print ( KBNODE keyblock, int secret, int fpr, void *opaque )
 	else if(pk->has_expired)
 	  {
 	    printf(" [");
-	    printf(_("expired: %s)"),expirestr_from_pk(pk));
+	    printf(_("expired: %s"),expirestr_from_pk(pk));
 	    printf("]");
 	  }
 	else if(pk->expiredate)
