@@ -1,5 +1,5 @@
 /* keyring.c - keyring file handling
- * Copyright (C) 2001 Free Software Foundation, Inc.
+ * Copyright (C) 2001, 2004 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -15,7 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -190,25 +191,31 @@ update_offset_hash_table_from_kb (OffsetHashTable tbl, KBNODE node, off_t off)
     }
 }
 
-
-
-
 /* 
- * Register a filename for plain keyring files.  Returns a pointer to
- * be used to create a handles etc or NULL to indicate that it has
- * already been registered */
-void *
-keyring_register_filename (const char *fname, int secret)
+ * Register a filename for plain keyring files.  ptr is set to a
+ * pointer to be used to create a handles etc, or the already-issued
+ * pointer if it has already been registered.  The function returns 1
+ * if a new keyring was registered.
+*/
+int
+keyring_register_filename (const char *fname, int secret, void **ptr)
 {
     KR_NAME kr;
 
     if (active_handles)
         BUG (); /* We don't allow that */
 
-    for (kr=kr_names; kr; kr = kr->next) {
+    for (kr=kr_names; kr; kr = kr->next)
+      {
         if ( !compare_filenames (kr->fname, fname) )
-            return NULL; /* already registered */
-    }
+	  {
+            *ptr=kr;
+	    return 0; /* already registered */
+	  }
+      }
+
+    if (secret)
+      register_secured_file (fname);
 
     kr = m_alloc (sizeof *kr + strlen (fname));
     strcpy (kr->fname, fname);
@@ -224,7 +231,9 @@ keyring_register_filename (const char *fname, int secret)
     if (!kr_offtbl)
       kr_offtbl = new_offset_hash_table ();
 
-    return kr;
+    *ptr=kr;
+
+    return 1;
 }
 
 int
@@ -363,10 +372,11 @@ keyring_get_keyblock (KEYRING_HANDLE hd, KBNODE *ret_kb)
         return -1; /* no successful search */
 
     a = iobuf_open (hd->found.kr->fname);
-    if (!a) {
-	log_error ("can't open `%s'\n", hd->found.kr->fname);
+    if (!a)
+      {
+	log_error(_("can't open `%s'\n"), hd->found.kr->fname);
 	return G10ERR_KEYRING_OPEN;
-    }
+      }
 
     if (iobuf_seek (a, hd->found.offset) ) {
         log_error ("can't seek `%s'\n", hd->found.kr->fname);
@@ -648,10 +658,11 @@ prepare_search (KEYRING_HANDLE hd)
 
     hd->current.eof = 0;
     hd->current.iobuf = iobuf_open (hd->current.kr->fname);
-    if (!hd->current.iobuf) {
-        log_error ("can't open `%s'\n", hd->current.kr->fname );
+    if (!hd->current.iobuf)
+      {
+        log_error(_("can't open `%s'\n"), hd->current.kr->fname );
         return (hd->current.error = G10ERR_OPEN_FILE);
-    }
+      }
 
     return 0;
 }
@@ -847,7 +858,8 @@ compare_name (int mode, const char *name, const char *uid, size_t uidlen)
  * for a keyblock which contains one of the keys described in the DESC array.
  */
 int 
-keyring_search (KEYRING_HANDLE hd, KEYDB_SEARCH_DESC *desc, size_t ndesc)
+keyring_search (KEYRING_HANDLE hd, KEYDB_SEARCH_DESC *desc,
+		size_t ndesc, size_t *descindex)
 {
   int rc;
   PACKET pkt;
@@ -1069,11 +1081,15 @@ keyring_search (KEYRING_HANDLE hd, KEYDB_SEARCH_DESC *desc, size_t ndesc)
 	}
       free_packet (&pkt);
       continue;
-    found:  
+    found:
+      /* Record which desc we matched on.  Note this value is only
+	 meaningful if this function returns with no errors. */
+      if(descindex)
+	*descindex=n;
       for (n=any_skip?0:ndesc; n < ndesc; n++) 
         {
           if (desc[n].skipfnc
-              && desc[n].skipfnc (desc[n].skipfncvalue, aki))
+              && desc[n].skipfnc (desc[n].skipfncvalue, aki, uid))
             break;
         }
       if (n == ndesc)
@@ -1172,14 +1188,21 @@ create_tmp_file (const char *template,
 
     /* Create the temp file with limited access */
     oldmask=umask(077);
-    *r_fp = iobuf_create (tmpfname);
+    if (is_secured_filename (tmpfname))
+      {
+        *r_fp = NULL;
+        errno = EPERM;
+      }
+    else
+      *r_fp = iobuf_create (tmpfname);
     umask(oldmask);
-    if (!*r_fp) {
-	log_error ("can't create `%s': %s\n", tmpfname, strerror(errno) );
+    if (!*r_fp)
+      {
+	log_error(_("can't create `%s': %s\n"), tmpfname, strerror(errno) );
         m_free (tmpfname);
         m_free (bakfname);
 	return G10ERR_OPEN_FILE;
-    }
+      }
     
     *r_bakfname = bakfname;
     *r_tmpfname = tmpfname;
@@ -1216,10 +1239,13 @@ rename_tmp_file (const char *bakfname, const char *tmpfname,
 #if defined(HAVE_DOSISH_SYSTEM) || defined(__riscos__)
   remove( fname );
 #endif
+  if (secret)
+    unregister_secured_file (fname);
   if (rename (tmpfname, fname) )
     {
-      log_error ("renaming `%s' to `%s' failed: %s\n",
+      log_error (_("renaming `%s' to `%s' failed: %s\n"),
                  tmpfname, fname, strerror(errno) );
+      register_secured_file (fname);
       rc = G10ERR_RENAME_FILE;
       if (secret)
         {
@@ -1300,7 +1326,7 @@ write_keyblock (IOBUF fp, KBNODE keyblock)
  * This is only done for the public keyrings.
  */
 int
-keyring_rebuild_cache (void *token)
+keyring_rebuild_cache (void *token,int noisy)
 {
   KEYRING_HANDLE hd;
   KEYDB_SEARCH_DESC desc;
@@ -1320,7 +1346,7 @@ keyring_rebuild_cache (void *token)
   if(rc)
     goto leave;
 
-  while ( !(rc = keyring_search (hd, &desc, 1)) )
+  while ( !(rc = keyring_search (hd, &desc, 1, NULL)) )
     {
       desc.mode = KEYDB_SEARCH_MODE_NEXT;
       resname = keyring_get_resource_name (hd);
@@ -1346,8 +1372,8 @@ keyring_rebuild_cache (void *token)
           if (rc)
             goto leave;
           lastresname = resname;
-          if (!opt.quiet)
-            log_info (_("checking keyring `%s'\n"), resname);
+          if (noisy && !opt.quiet)
+            log_info (_("caching keyring `%s'\n"), resname);
           rc = create_tmp_file (resname, &bakfilename, &tmpfilename, &tmpfp);
           if (rc)
             goto leave;
@@ -1365,16 +1391,24 @@ keyring_rebuild_cache (void *token)
       /* check all signature to set the signature's cache flags */
       for (node=keyblock; node; node=node->next)
         {
+	  /* Note that this doesn't cache the result of a revocation
+	     issued by a designated revoker.  This is because the pk
+	     in question does not carry the revkeys as we haven't
+	     merged the key and selfsigs.  It is questionable whether
+	     this matters very much since there are very very few
+	     designated revoker revocation packets out there. */
+
           if (node->pkt->pkttype == PKT_SIGNATURE)
             {
-	      /* Note that this doesn't cache the result of a
-		 revocation issued by a designated revoker.  This is
-		 because the pk in question does not carry the revkeys
-		 as we haven't merged the key and selfsigs.  It is
-		 questionable whether this matters very much since
-		 there are very very few designated revoker revocation
-		 packets out there. */
-              check_key_signature (keyblock, node, NULL);
+	      PKT_signature *sig=node->pkt->pkt.signature;
+
+	      if(!opt.no_sig_cache && sig->flags.checked && sig->flags.valid
+		 && (check_digest_algo(sig->digest_algo)
+		     || check_pubkey_algo(sig->pubkey_algo)))
+		sig->flags.checked=sig->flags.valid=0;
+	      else
+		check_key_signature (keyblock, node, NULL);
+
               sigcount++;
             }
         }
@@ -1384,8 +1418,8 @@ keyring_rebuild_cache (void *token)
       if (rc)
         goto leave;
 
-      if ( !(++count % 50) && !opt.quiet)
-        log_info(_("%lu keys so far checked (%lu signatures)\n"),
+      if ( !(++count % 50) && noisy && !opt.quiet)
+        log_info(_("%lu keys cached so far (%lu signatures)\n"),
                  count, sigcount );
 
     } /* end main loop */ 
@@ -1396,7 +1430,8 @@ keyring_rebuild_cache (void *token)
       log_error ("keyring_search failed: %s\n", g10_errstr(rc));
       goto leave;
     }
-  log_info(_("%lu keys checked (%lu signatures)\n"), count, sigcount );
+  if(noisy || opt.verbose)
+    log_info(_("%lu keys cached (%lu signatures)\n"), count, sigcount );
   if (tmpfp)
     {
       if (iobuf_close (tmpfp))
@@ -1442,7 +1477,7 @@ do_copy (int mode, const char *fname, KBNODE root, int secret,
     char *bakfname = NULL;
     char *tmpfname = NULL;
 
-    /* Open the source file. Because we do a rname, we have to check the 
+    /* Open the source file. Because we do a rename, we have to check the 
        permissions of the file */
     if (access (fname, W_OK))
       return G10ERR_WRITE_FILE;
@@ -1454,13 +1489,18 @@ do_copy (int mode, const char *fname, KBNODE root, int secret,
 	mode_t oldmask;
 
 	oldmask=umask(077);
-	newfp = iobuf_create (fname);
+        if (!secret && is_secured_filename (fname)) {
+            newfp = NULL;
+            errno = EPERM;
+        }
+        else
+            newfp = iobuf_create (fname);
 	umask(oldmask);
-	if( !newfp ) {
-	    log_error (_("%s: can't create: %s\n"),
-                       fname, strerror(errno));
+	if( !newfp )
+	  {
+	    log_error (_("can't create `%s': %s\n"), fname, strerror(errno));
 	    return G10ERR_OPEN_FILE;
-	}
+	  }
 	if( !opt.quiet )
 	    log_info(_("%s: keyring created\n"), fname );
 
@@ -1480,18 +1520,22 @@ do_copy (int mode, const char *fname, KBNODE root, int secret,
 	return 0; /* ready */
     }
 
-    if( !fp ) {
-	log_error ("%s: can't open: %s\n", fname, strerror(errno) );
+    if( !fp )
+      {
+	log_error(_("can't open `%s': %s\n"), fname, strerror(errno) );
 	rc = G10ERR_OPEN_FILE;
 	goto leave;
-    }
+      }
 
-    /* create the new file */
+    /* Create the new file.  */
     rc = create_tmp_file (fname, &bakfname, &tmpfname, &newfp);
     if (rc) {
 	iobuf_close(fp);
 	goto leave;
     }
+    if (secret)
+      register_secured_file (tmpfname);
+
     if( mode == 1 ) { /* insert */
 	/* copy everything to the new file */
 	rc = copy_all_packets (fp, newfp);
@@ -1499,6 +1543,8 @@ do_copy (int mode, const char *fname, KBNODE root, int secret,
 	    log_error("%s: copy to `%s' failed: %s\n",
 		      fname, tmpfname, g10_errstr(rc) );
 	    iobuf_close(fp);
+            if (secret)
+              unregister_secured_file (tmpfname);
 	    iobuf_cancel(newfp);
 	    goto leave;
 	}
@@ -1512,6 +1558,8 @@ do_copy (int mode, const char *fname, KBNODE root, int secret,
 	    log_error ("%s: copy to `%s' failed: %s\n",
                        fname, tmpfname, g10_errstr(rc) );
 	    iobuf_close(fp);
+            if (secret)
+              unregister_secured_file (tmpfname);
 	    iobuf_cancel(newfp);
 	    goto leave;
 	}
@@ -1522,6 +1570,8 @@ do_copy (int mode, const char *fname, KBNODE root, int secret,
 	    log_error("%s: skipping %u packets failed: %s\n",
 			    fname, n_packets, g10_errstr(rc));
 	    iobuf_close(fp);
+            if (secret)
+              unregister_secured_file (tmpfname);
 	    iobuf_cancel(newfp);
 	    goto leave;
 	}
@@ -1531,6 +1581,8 @@ do_copy (int mode, const char *fname, KBNODE root, int secret,
         rc = write_keyblock (newfp, root);
         if (rc) {
           iobuf_close(fp);
+          if (secret)
+            unregister_secured_file (tmpfname);
           iobuf_cancel(newfp);
           goto leave;
         }
@@ -1543,6 +1595,8 @@ do_copy (int mode, const char *fname, KBNODE root, int secret,
 	    log_error("%s: copy to `%s' failed: %s\n",
 		      fname, tmpfname, g10_errstr(rc) );
 	    iobuf_close(fp);
+            if (secret)
+              unregister_secured_file (tmpfname);
 	    iobuf_cancel(newfp);
 	    goto leave;
 	}
