@@ -1,5 +1,6 @@
 /* openfile.c
- * Copyright (C) 1998, 1999, 2000, 2001, 2003 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004,
+ *               2005 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -15,7 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -147,8 +149,10 @@ ask_outfile_name( const char *name, size_t namelen )
 	sprintf(prompt, "%s [%s]: ", s, defname );
     else
 	sprintf(prompt, "%s: ", s );
+    tty_enable_completion(NULL);
     fname = cpr_get("openfile.askoutname", prompt );
     cpr_kill_prompt();
+    tty_disable_completion();
     m_free(prompt);
     if( !*fname ) {
 	m_free( fname ); fname = NULL;
@@ -263,6 +267,9 @@ open_outfile( const char *iname, int mode, IOBUF *a )
     m_free(buf);
   }
 
+  if (*a)
+    iobuf_ioctl (*a,3,1,NULL); /* disable fd caching */
+
   return rc;
 }
 
