@@ -1,5 +1,5 @@
 /* seskey.c -  make sesssion keys etc.
- * Copyright (C) 1998, 1999, 2000, 2001, 2004 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -15,7 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -117,13 +118,13 @@ encode_session_key( DEK *dek, unsigned nbits )
 		k++;
 	if( !k )
 	    break; /* okay: no zero bytes */
-	k += (k/128) + 3; /* better get some more */
+	k += k/128 + 3; /* better get some more */
 	pp = get_random_bits( k*8, 1, 1);
-	for (j=0; j < i && k; ) {
-          if ( !p[j] )
-            p[j] = pp[--k];
-          if (p[j])
-            j++;
+	for(j=0; j < i && k ;) {
+	    if( !p[j] )
+		p[j] = pp[--k];
+            if (p[j])
+              j++;
         }
 	m_free(pp);
     }
@@ -145,7 +146,7 @@ encode_session_key( DEK *dek, unsigned nbits )
 
 static MPI
 do_encode_md( MD_HANDLE md, int algo, size_t len, unsigned nbits,
-	      const byte *asn, size_t asnlen, int v3compathack )
+	      const byte *asn, size_t asnlen )
 {
     int nframe = (nbits+7) / 8;
     byte *frame;
@@ -158,14 +159,14 @@ do_encode_md( MD_HANDLE md, int algo, size_t len, unsigned nbits,
 
     /* We encode the MD in this way:
      *
-     *	   0  A PAD(n bytes)   0  ASN(asnlen bytes)  MD(len bytes)
+     *	   0  1 PAD(n bytes)   0  ASN(asnlen bytes)  MD(len bytes)
      *
      * PAD consists of FF bytes.
      */
     frame = md_is_secure(md)? m_alloc_secure( nframe ) : m_alloc( nframe );
     n = 0;
     frame[n++] = 0;
-    frame[n++] = v3compathack? algo : 1; /* block type */
+    frame[n++] = 1; /* block type */
     i = nframe - len - asnlen -3 ;
     assert( i > 1 );
     memset( frame+n, 0xff, i ); n += i;
@@ -199,8 +200,8 @@ do_encode_md( MD_HANDLE md, int algo, size_t len, unsigned nbits,
  * the encoded value.  Setting this flag forces the old behaviour.
  */
 MPI
-encode_md_value( int pubkey_algo, MD_HANDLE md, int hash_algo,
-		 unsigned nbits, int v3compathack )
+encode_md_value( int pubkey_algo, MD_HANDLE md,
+		 int hash_algo, unsigned nbits )
 {
     int algo = hash_algo? hash_algo : md_get_algo(md);
     const byte *asn;
@@ -223,8 +224,7 @@ encode_md_value( int pubkey_algo, MD_HANDLE md, int hash_algo,
     }
     else {
        asn = md_asn_oid( algo, &asnlen, &mdlen );
-       frame = do_encode_md( md, algo, mdlen, nbits, asn, asnlen, v3compathack);
+       frame = do_encode_md( md, algo, mdlen, nbits, asn, asnlen );
     }
     return frame;
 }
-
