@@ -1,6 +1,6 @@
 /* status.c
  * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003,
- *               2004 Free Software Foundation, Inc.
+ *               2004, 2005 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -16,7 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -159,6 +160,9 @@ get_status_string ( int no )
     case STATUS_PLAINTEXT      : s = "PLAINTEXT"; break;
     case STATUS_PLAINTEXT_LENGTH:s = "PLAINTEXT_LENGTH"; break;
     case STATUS_SIG_SUBPACKET  : s = "SIG_SUBPACKET"; break;
+    case STATUS_SC_OP_SUCCESS  : s = "SC_OP_SUCCESS"; break;
+    case STATUS_SC_OP_FAILURE  : s = "SC_OP_FAILURE"; break;
+    case STATUS_BACKUP_KEY_CREATED:s="BACKUP_KEY_CREATED"; break;
     default: s = "?"; break;
     }
   return s;
@@ -259,7 +263,8 @@ write_status_text ( int no, const char *text)
         }
     }
     putc ('\n',statusfp);
-    fflush (statusfp);
+    if ( fflush (statusfp) && opt.exit_on_status_write_error )
+      g10_exit (0);
 }
 
 
@@ -323,7 +328,8 @@ write_status_text_and_buffer ( int no, const char *string,
     } while ( len );
 
     putc ('\n',statusfp);
-    fflush (statusfp);
+    if ( fflush (statusfp) && opt.exit_on_status_write_error )
+      g10_exit (0);
 }
 
 void
