@@ -16,7 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 #ifndef G10_STATUS_H
 #define G10_STATUS_H
@@ -107,6 +108,14 @@
 #define STATUS_NEED_PASSPHRASE_PIN 76
 #define STATUS_SIG_SUBPACKET    77
 
+/* Extra status codes for certain smartcard operations.  Primary
+   useful to double check that change PIN worked as expected.  */
+#define STATUS_SC_OP_FAILURE    79
+#define STATUS_SC_OP_SUCCESS    80
+
+#define STATUS_BACKUP_KEY_CREATED 81
+
+
 /*-- status.c --*/
 void set_status_fd ( int fd );
 int  is_status_enabled ( void );
