@@ -1,5 +1,5 @@
 /* memory.h - memory allocation
- *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2005 Free Software Foundation, Inc.
  *
  * This file is part of GNUPG.
  *
@@ -15,7 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #ifndef G10_MEMORY_H
@@ -66,7 +67,7 @@ size_t m_size( const void *a );
 void m_print_stats(const char *prefix);
 
 /*-- secmem.c --*/
-void secmem_init( size_t npool );
+int secmem_init( size_t npool );
 void secmem_term( void );
 void *secmem_malloc( size_t size );
 void *secmem_realloc( void *a, size_t newsize );
@@ -90,4 +91,15 @@ unsigned secmem_get_flags(void);
 EXTERN_UNLESS_MAIN_MODULE int memory_debug_mode;
 EXTERN_UNLESS_MAIN_MODULE int memory_stat_debug_mode;
 
+/* To prepare a migration to the xmalloc suite of function as used in
+   1.9 we define a couple of macros. */
+#define xmalloc(n)        m_alloc ((n))
+void *xcalloc (size_t n, size_t m);
+#define xmalloc_secure(n) m_alloc_secure (n)
+void *xcalloc_secure (size_t n, size_t m);
+#define xrealloc(a,n)     m_realloc ((a),(n))
+#define xstrdup(a)        m_strdup ((a))
+#define xfree(a)          m_free (a)
+
+
 #endif /*G10_MEMORY_H*/
