@@ -15,7 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -33,6 +34,8 @@
 #include "keyserver.h"
 #include "ksutil.h"
 
+#define GET    0
+
 extern char *optarg;
 extern int optind;
 
@@ -45,12 +48,6 @@ static char port[MAX_PORT+1];
 static char path[URLMAX_PATH+1];
 static FILE *input,*output,*console;
 
-#ifdef __riscos__
-#define HTTP_PROXY_ENV           "GnuPG$HttpProxy"
-#else
-#define HTTP_PROXY_ENV           "http_proxy"
-#endif
-
 static int
 get_key(char *getkey)
 {
@@ -70,10 +67,11 @@ get_key(char *getkey)
       return KEYSERVER_NO_MEMORY;
     }
 
-  sprintf(request,"http://%s%s%s%s%s%s%s",auth[0]?auth:"",auth[0]?"@":"",
-	  host,port[0]?":":"",port[0]?port:"",path[0]?"":"/",path);
+  sprintf(request,"http://%s%s%s%s%s",host,
+	  port[0]?":":"",port[0]?port:"",path[0]?"":"/",path);
 
-  rc=http_open_document(&hd,request,http_flags,proxy[0]?proxy:NULL);
+  rc=http_open_document(&hd,request,auth[0]?auth:NULL,
+			http_flags,proxy[0]?proxy:NULL);
   if(rc!=0)
     {
       fprintf(console,"gpgkeys: HTTP fetch error: %s\n",
@@ -92,14 +90,14 @@ get_key(char *getkey)
 
 	  if(gotit)
 	    {
-	      fputs(line,output);
+	      print_nocr(output,line);
 	      if(strncmp(line,END,strlen(END))==0)
 		break;
 	    }
 	  else
 	    if(strncmp(line,BEGIN,strlen(BEGIN))==0)
 	      {
-		fputs(line,output);
+		print_nocr(output,line);
 		gotit=1;
 	      }
 	}
